from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker  # Updated import
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict

from actions.helpers.openai_API import (
    OpenAI
)

class CompleteAskOpenAIForm(Action):
    """
       Query open API and respond to user
    """
    def name(self) -> Text:
        return "action_complete_ask_openai_form"

    async def run(  # Updated to async
        self,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict
    ) -> List[EventType]:
        '''
            Uses the tracker to get the required slots
        '''

        # extract user question
        user_message_openai = tracker.get_slot("question_openai")
        
        # Generate a response from OpenAI
        openai_instance = OpenAI()

        response = await openai_instance.ask_openai(user_message_openai)  # Added await

        if response is None:
            dispatcher.utter_message(
                response="connection_error",
                service_name="OpenAI"
            )
        else:
            # send response to user
            dispatcher.utter_message(response)
        
        # clear slot
        return [SlotSet("question_openai", None)]