from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher # this was commented out
from rasa_sdk.events import (
    SlotSet,
    EventType,
    ActionExecuted,
    SessionStarted
)

# Needed to import graphql helper functions
import actions.helpers.graphql_helpers as helpers

import logging
log = logging.getLogger("Debugging")


class ActionSessionStart(Action):
    def name(self) -> Text:
        return "action_session_start"

    @staticmethod
    def fetch_slots(tracker: Tracker) -> List[EventType]:
        """Fetches SlotSet events from tracker and carries over keys and values"""

        # when restarting most slots should be reset
        slots = []
        # for key in ("user_id"):
        #     value = tracker.get_slot(key)
        #     if value is not None:
        #         slots.append(SlotSet(key=key, value=value))
        return slots

    async def run(
      self, dispatcher, tracker: Tracker, domain: Dict[Text, Any]
    ) -> List[Dict[Text, Any]]:

        # the session should begin with a `session_started` event
        events = [SessionStarted()]

        # any slots that should be carried over should come after the
        # `session_started` event
        events.extend(self.fetch_slots(tracker))

        # find user profile info and fill the slots
        # user_id = tracker.get_slot("user_id")

        # TODO When graphql resolvers are ready, we can uncomment the lines in this function
        #user_profile_slot_events = helpers.get_user_profile(dispatcher, user_id)
        # TODO What if the user id is not valid?

        # initialize slots from user profile
        #events = events + user_profile_slot_events

        # an `action_listen` should be added at the end as a user message follows
        events.append(ActionExecuted("action_listen"))

        return events


class ActionRestart(Action):
    """Executes after restart of a session"""

    def name(self) -> Text:
        """Unique identifier of the action"""
        return "action_restart"

    async def run(
        self,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: Dict[Text, Any],
    ) -> List[EventType]:

        return [
                Restarted()
            ]
