from typing import Dict, Text, List, Optional, Any

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
from rasa_sdk.forms import FormValidationAction
# from rasa_sdk.forms import FormValidationAction

# Needed to import synonym helper function
from actions.helpers.synonym_manager import (
   find_synonym_for_agnostic_entity
   )
# Needed to interact with graphql database
# from actions.graphql_interfaces.graphql_queries import (
#    graphql_get_product_profile_active_checklist_step,
#    graphql_update_product_profile
#    )

# Needed to import graphql helper functions
# import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)

class actionCompleteIssueSupport1(Action):
    """
        After user confirms the recorded information, the bot saves
        the slots, writes them in the database, and resets them.
    """

    def name(self) -> Text:
        return "action_complete_issue_support"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        user_action = tracker.get_slot('user_action')
        machine_component_long = tracker.get_slot('machine_component')
        machine_component = machine_component_long.replace('MC','')
        machine_component_state = tracker.get_slot('machine_component_state')
        product_component = tracker.get_slot('product_component')
        product_component_state = tracker.get_slot('product_component_state')

        return [SlotSet("report_symptom_machine_component",machine_component),
                SlotSet("report_symptom_machine_component_state",machine_component_state),
                SlotSet("report_symptom_product_component",product_component),
                SlotSet("report_symptom_product_component_state",product_component_state),
                SlotSet("report_symptom_user_action",user_action),
                SlotSet("user_action", None),
                SlotSet("machine_component", None),
                SlotSet("machine_component_state", None),
                SlotSet("product_component", None),
                SlotSet("product_component_state", None)]


class actionCompleteIssueSupport(Action):
    """
        After user confirms the recorded information, the bot saves
        the slots, writes them in the database, and resets them.
    """

    def name(self) -> Text:
        return "action_complete_issue_support"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:
        
        dispatcher.utter_message(response="utter_issue_solution_requested")

        return [SlotSet("product_number", None),
                SlotSet("report_symptom_description", None),
                SlotSet("report_symptom_machine_component", None),
                SlotSet("report_symptom_machine_component_state", None),
                SlotSet("report_symptom_product_component", None),
                SlotSet("report_symptom_product_component_state", None)]