from typing import Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict

#import actions.helpers.config_loader as config
import actions.helpers.gitlab_ticket_uploader as gitlab_ticket_uploader

import requests
#from requests.exceptions import ConnectionError

import logging
log = logging.getLogger("Gitlab general ticket creation")

class CreateGitlabTicket(Action):
    """
        Creates an issue/ticket from slots.
        The user can provide one or two statements. The second statment
        allows users to provide more details.
        After filling the ticket_description slot, the latest message
        becomes the second statement.
    """
    def name(self) -> Text:
        return "action_create_gitlab_ticket"

    def run(
        self,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict
    ) -> List[EventType]:
        '''
            Uses the tracker to get the required slots
        '''

        gitlab_ticket_description = tracker.get_slot('gitlab_ticket_description')
        gitlab_ticket_title = tracker.get_slot('gitlab_ticket_title')

        # latest_message = get_latest_user_message(tracker)
        # description = f"{ticket_description} {latest_message}"
        # log.info(f"Clause: {ticket_description}")

        issue = gitlab_ticket_uploader.Issue(title=gitlab_ticket_title, description=gitlab_ticket_description)
        response: requests.Response = issue.send()

        if response.ok:
            dispatcher.utter_message(response='utter_gitlab_ticket_created')
        else:
            dispatcher.utter_message(
                response="connection_error",
                service_name="gitlab"
            )
        # Resets the slot after issue creation.
        return [
            SlotSet('gitlab_ticket_description', None),
            SlotSet('gitlab_ticket_title', None)
        ]