from typing import Text, Dict, List

from rasa_sdk import Action, Tracker  # Updated import
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict

#import actions.helpers.config_loader as config

from requests.exceptions import ConnectionError


from actions.helpers.neo4jQueries import (
   Neo4jQuery
   )

from actions.helpers.openai_API import (
    OpenAI
)

# Needed to import graphql helper functions
import actions.helpers.graphql_helpers as helpers

import logging
from datetime import datetime
log = logging.getLogger("providing issue support")

class ProvideIssueSupport(Action):
    """
     description
    """
    def name(self) -> Text:
        return "action_provide_issue_support"

    async def run(  # Updated to async
        self,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict
    ) -> List[EventType]:
        '''
            Uses the tracker to get the required slots
        '''
        # machine_component_long = tracker.get_slot('machine_component')
        # machine_component = machine_component_long.replace('MC','')
        # machine_component_state = tracker.get_slot('machine_component_state')
        # product_component = tracker.get_slot('product_component')
        # product_component_state = tracker.get_slot('product_component_state')
        # user_action = tracker.get_slot('user_action')
        symptoms = tracker.get_slot('report_symptom_description')
        # descEn = datetime.now().strftime("%H:%M:%S")

        # First check neo4j for possible causes and solutions

        error_code = tracker.get_slot('error_code')

        neo4j_query = Neo4jQuery()
        
        status, result = neo4j_query.get_solutions(error_code)

        if status == "success":
            prompt = f"Dit zijn de symptomen: {symptoms}. Het betreft foutcode {error_code}. In de database heb ik de volgende mogelijke oplossing voor deze foutcode gevonden: {result}. Wat moet ik doen?"
        else:
            prompt = symptoms

        openai_instance = OpenAI()

        response = await openai_instance.ask_openai(prompt)  # Added await

        if response is None:
            dispatcher.utter_message(
                response="connection_error",
                service_name="OpenAI"
            )
        else:
            # send response to user
            dispatcher.utter_message(response)

        return [# SlotSet("report_symptom_product_number",product_number),
                SlotSet("user_action", None),
                SlotSet("error_code", None),
                SlotSet("machine_component", None),
                SlotSet("machine_component_state", None),
                SlotSet("product_component", None),
                SlotSet("product_component_state", None),
                SlotSet("report_symptom_description", None),
                SlotSet("product_number", None)]