from typing import Text, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
from datetime import datetime

from actions.helpers.neo4jQueries import (
   Neo4jQuery
   )

#import actions.helpers.config_loader as config
import actions.helpers.gitlab_ticket_uploader as gitlab_ticket_uploader

import requests
#from requests.exceptions import ConnectionError


# from rasa_sdk.forms import FormValidationAction

# Needed to import synonym helper function
from actions.helpers.synonym_manager import (
    find_synonym_for_agnostic_entity
    )

import logging
logger = logging.getLogger(__name__)

class actionUtterNewSKUFillerSettingsRecorded(Action):
    """
        once the form is filled, the bot utters
        the recorded information and asks for confirmation
    """

    def name(self) -> Text:
        return "action_utter_new_SKU_filler_settings_recorded"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]: 

        product_number = tracker.get_slot("product_number")
        filling_speed = tracker.get_slot("filling_speed")
        filling_pressure = tracker.get_slot("filling_pressure")
        comments = tracker.get_slot("comments")
        # user_id = tracker.get_slot("user_id")

        dt = datetime.now()
        unix_timestamp = int(dt.timestamp())

        # machine_component = "10600"

        # title = f"New filling settings for {product_number} | Settings ID: {current_time}"
        # description = f"Filling pressure: {filling_pressure}\nFilling speed: {filling_speed}\nComments: {comments}\nSKU: {product_number}\nSettings ID: {current_time}"

        # # synonym_product_parameter_value = find_synonym_for_agnostic_entity(
        # #                        product_parameter_value)["synonym_for_entity"]

        # # create gitlab ticket
        # issue = gitlab_ticket_uploader.Issue(title=title, description=description)
        # response: requests.Response = issue.send()

        # remove "product " that precedes the number
        # product_number_short = product_number[8:]

        # settings_name = current_time

        # Create an instance of the Neo4jQuery class
        neo4j_query = Neo4jQuery()

        # Call the create_settings_node function with example parameters
        batch_stage = "unknown"
        rating = 5
        rating_count = 1
        approval = "unapproved"
        # try:

        result_status, result_data = neo4j_query.create_settings_node(filling_pressure, filling_speed, batch_stage, unix_timestamp, comments, rating, rating_count, approval, product_number)

        if result_status == "success":
            dispatcher.utter_message(
                    response="utter_new_SKU_filler_settings_recorded",
                    product_number=product_number,
                    filling_speed=filling_speed,
                    filling_pressure=filling_pressure,
                    comments=comments) 
        else:
            print("Failed to create settings node.")
            print("Error message:", result_data)
            dispatcher.utter_message(
                response="connection_error",
                service_name="Neo4j"
            )

        return []

class actionUtterNewSKUFillerSettingsSaved(Action):
    """
        After user confirms the recorded information, the bot saves
        the slots, writes them in the database, and resets them.
    """

    def name(self) -> Text:
        return "action_utter_new_SKU_filler_settings_saved"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:
        
        dispatcher.utter_message(response="utter_new_SKU_filler_settings_registered")

        return [SlotSet("product_number", None),
                SlotSet("filling_speed", None),
                SlotSet("comments", None),
                SlotSet("filling_pressure", None)]

