from typing import Text, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
# from rasa_sdk.forms import FormValidationAction

# Needed to import synonym helper function
# from actions.helpers.synonym_manager import (
#    find_synonym_for_agnostic_entity
#    )
# Needed to interact with graphql database
# from actions.graphql_interfaces.graphql_queries import (
#    graphql_get_product_profile_active_checklist_step,
#    graphql_update_product_profile
#    )

# Needed to import graphql helper functions
# import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)


class actionUtterNewSKUWashoutRecorded(Action):
    """
        once the form is filled, the bot utters
        the recorded information and asks for confirmation
    """

    def name(self) -> Text:
        return "action_utter_new_SKU_washout_recorded"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        previous_product_number = tracker.get_slot("previous_product_number")
        next_product_number = tracker.get_slot("next_product_number")
        washout_duration = tracker.get_slot("washout_duration")
        washout_pressure = tracker.get_slot("washout_pressure")
        washout_temperature = tracker.get_slot("washout_temperature")
        washout_cycles = tracker.get_slot("washout_cycles")
        comments = tracker.get_slot("comments")
        # synonym_product_parameter_value = find_synonym_for_agnostic_entity(
        #                        product_parameter_value)["synonym_for_entity"]

        dispatcher.utter_message(
                response="utter_new_SKU_washout_recorded",
                previous_product_number=previous_product_number,
                next_product_number=next_product_number,
                washout_duration=washout_duration,
                washout_pressure=washout_pressure,
                washout_temperature=washout_temperature,
                washout_cycles=washout_cycles,
                comments=comments)

        return []


class actionUtterNewSKUWashoutSaved(Action):
    """
        After user confirms the recorded information, the bot saves
        the slots, writes them in the database, and resets them.
    """

    def name(self) -> Text:
        return "action_utter_new_SKU_washout_saved"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        # TODO write the defect information to the database
        # TODO user information to get from user profile
        # TODO try to find the defect code from DCS, if not available
        # TODO then get the component + defect from the slots
        
        dispatcher.utter_message(response="utter_new_SKU_washout_registered")

        return [SlotSet("next_product_number", None),
                SlotSet("previous_product_number", None),
                SlotSet("washout_duration", None),
                SlotSet("washout_pressure", None),
                SlotSet("washout_temperature", None),
                SlotSet("washout_cycles", None),
                SlotSet("comments", None)]

