from typing import Dict, Text, List, Optional, Any

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
from rasa_sdk.forms import FormValidationAction
# from rasa_sdk.forms import FormValidationAction
import datetime

from actions.graphql_interfaces.graphql_queries import (
    graphql_createEvMcConn,
    graphql_createEvent,
    graphql_createErrCode,
    graphql_createEvMcConn,
    graphql_createEvent,
    graphql_createRateSol,
    graphql_createSolutionConn,
    graphql_createStatSol
)

# from actions.helpers.lab_sim_query import (
#    get_status
#    )

# Needed to import synonym helper function
from actions.helpers.synonym_manager import (
   find_synonym_for_agnostic_entity
   )
# Needed to interact with graphql database
# from actions.graphql_interfaces.graphql_queries import (
#    graphql_get_product_profile_active_checklist_step,
#    graphql_update_product_profile
#    )

# Needed to import graphql helper functions
# import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)

class actionCompleteIssueSymptomReport(Action):
    """
        After user confirms the recorded information, the bot saves
        the slots, writes them in the database, and resets them.
    """

    def name(self) -> Text:
        return "action_complete_issue_symptom_report"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:
        
        #dispatcher.utter_message(response="utter_new_issue_registered")

        # user_action = tracker.get_slot('user_action')
        error_code = tracker.get_slot('error_code')
        machine_component_long = tracker.get_slot('machine_component')
        machine_component = machine_component_long.replace('MC','')
        # machine_component_state = tracker.get_slot('machine_component_state')
        # product_component = tracker.get_slot('product_component')
        # product_component_state = tracker.get_slot('product_component_state')
        # product_number = tracker.get_slot('product_number')
        report_symptom_description = tracker.get_slot('report_symptom_description')
        report_solution_description = tracker.get_slot('report_solution_description')

        # set symptom description to current time is nothing is provided
        if report_symptom_description is None:
            now = datetime.datetime.now()
            report_symptom_description = now.strftime("%Y-%m-%d %H:%M:%S")
        
        # status_response = get_status()
        # product_number = status_response["product_number"]
        # machine_state = status_response["status"]
        # error_code = status_response["message"]

        try:
            graphql_createEvent(report_symptom_description,report_symptom_description)

            if error_code == "0":
                error_code = None
            else:
                graphql_createErrCode(error_code,report_symptom_description)

            graphql_createEvMcConn(report_symptom_description,"","","",machine_component)

            if report_solution_description is not None:
                graphql_createEvent(report_solution_description,report_solution_description)
                graphql_createSolutionConn(report_solution_description,report_symptom_description)
                # set initial solution status
                graphql_createStatSol(report_solution_description,"pending approval")
                # set initial solution rating
                graphql_createRateSol(report_solution_description,float(5))
 
        except:
            dispatcher.utter_message(
                response="connection_error",
                service_name="Neo4j"
            )  

        dispatcher.utter_message(response="utter_done")

        return [SlotSet("user_action", None),
                SlotSet("error_code", None),
                SlotSet("machine_component", None),
                SlotSet("machine_component_state", None),
                SlotSet("product_component", None),
                SlotSet("product_component_state", None),
                SlotSet("report_symptom_description", None),
                SlotSet("report_solution_description", None),
                SlotSet("require_new_solution", None)]
    
class ValidateIssueReportSymptomForm(FormValidationAction):
    """
        validate form
    """
    def name(self) -> Text:
        return "validate_issue_report_symptom_form"
    
    def validate_require_new_solution(
        self,
        slot_value: Any,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict,
    ) -> Dict[Text, Any]:
        
        # adding no validation jsut so that the require_slots function is activated
        return {"require_new_solution": slot_value}

    async def required_slots(
        self,
        domain_slots: List[Text],
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict,
    ) -> List[Text]:
        additional_slots = []

        # if bool(tracker.slots.get("report_solution_description")) and "require_new_solution" not in domain_slots:
        #     # the a solution wasn't extracted from the issue description, append a slot to ask if they want to add a solution
        #     additional_slots.append("require_new_solution")
        print("i did something")

        if tracker.slots.get("error_code")=="0" and "report_symptom_description" not in domain_slots:
            # if no error code was provided, ask for a symptom description instead
            additional_slots.append("report_symptom_description")

        if tracker.slots.get("require_new_solution") is True and "report_solution_description" not in domain_slots:
            # the user wants to add a new solution, we append the slot and ask for it.
            additional_slots.append("report_solution_description")

        return additional_slots + domain_slots
