from typing import Text, List
import random
from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
# from datetime import datetime
# from dateutil import tz
# from rasa_sdk.forms import FormValidationAction

# Needed to import synonym helper function
# from actions.helpers.synonym_manager import (
#    find_synonym_for_agnostic_entity
#    )
# Needed to interact with graphql database
# from actions.graphql_interfaces.graphql_queries import (
#     graphql_searchset
#     )

from actions.helpers.neo4jQueries import (
   Neo4jQuery
   )

# Needed to import graphql helper functions
import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)


class actionUtterSKUFillerSettingsRequest(Action):
    """
        once the form is filled, the bot utters
        the requested information
    """

    def name(self) -> Text:
        return "action_utter_SKU_filler_settings_requested"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        product_number = tracker.get_slot("product_number")
        
        # synonym_product_parameter_value = find_synonym_for_agnostic_entity(
        #                        product_parameter_value)["synonym_for_entity"]
        # filling_speed_value = random.randint(40,80)
        # filling_pressure_value = random.randint(350,600)
        # filling_speed = f"{filling_speed_value} RPM"
        # filling_pressure = f"{filling_pressure_value} millibar"

        # try:
            
        # Create an instance of the Neo4jQuery class
        neo4j_query = Neo4jQuery()

        # Example parameter for the get_settings_by_sku function

        # Call the get_settings_by_sku function with an example parameter
        result_status, result_data = neo4j_query.get_settings_by_sku(product_number)

        # Check if the operation was successful or not
        if result_status == "success":
            # Loop through the 3 latest results (or all results if less than 3)
            for settings in result_data[:3]:
                filling_pressure = settings["pressure"]
                filling_speed = settings["speed"]
                comments = settings["comments"]
                rate_rounded = round(settings["rating"])
                status = "Approved" if settings["approval"] else "Not Approved"
                time = settings["datetime"]

                # Dispatch the message in the specified format
                dispatcher.utter_message(response="utter_requested_settings",
                                        filling_pressure=filling_pressure,
                                        filling_speed=filling_speed,
                                        comments=comments,
                                        rate_rounded=rate_rounded,
                                        status=status,
                                        time=time)
        elif result_status == "error":
            print("Failed to retrieve settings.")
            print("Error message:", result_data)
            dispatcher.utter_message(response="utter_no_settings_found",product_number=product_number)
        elif result_status == "info":
            print(result_data)  # No settings found for the given SKU
            dispatcher.utter_message(response="utter_no_settings_found",product_number=product_number)
            
        return []

        # dispatcher.utter_message(
        #         response="utter_SKU_filler_settings_requested",
        #         product_number=product_number,
        #         filling_speed=filling_speed,
        #         filling_pressure=filling_pressure
        #         )
        #except:
        #    dispatcher.utter_message(f"Sorry, I couldn't find any settings for {product_number}")

    
class actionSKUFillerSettingsRequestCompleted(Action):
    """
        After user receives requested information the bot resets the slots.
    """

    def name(self) -> Text:
        return "action_SKU_filler_settings_request_completed"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        # TODO write the defect information to the database
        # TODO user information to get from user profile
        # TODO try to find the defect code from DCS, if not available
        # TODO then get the component + defect from the slots

        #dispatcher.utter_message(response="utter_SKU_filler_settings_request_registered")

        return [
            SlotSet("product_number", None)
        ]
