from typing import Text, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
# from rasa_sdk.forms import FormValidationAction

# Needed to import synonym helper function
# from actions.helpers.synonym_manager import (
#    find_synonym_for_agnostic_entity
#    )
# Needed to interact with graphql database
# from actions.graphql_interfaces.graphql_queries import (
#    graphql_get_product_profile_active_checklist_step,
#    graphql_update_product_profile
#    )

# Needed to import graphql helper functions
# import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)


class actionNewSKUWashoutRequested(Action):
    """
        once the form is filled, the bot utters
        the requested information
    """

    def name(self) -> Text:
        return "action_utter_SKU_washout_requested"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        previous_product_number = tracker.get_slot("previous_product_number")
        next_product_number = tracker.get_slot("next_product_number")
        washout_duration = "5 minutes"
        washout_pressure = "2 bar"
        washout_temperature = "56 celcius"
        washout_cycles = "5"
        # synonym_product_parameter_value = find_synonym_for_agnostic_entity(
        #                        product_parameter_value)["synonym_for_entity"]

        dispatcher.utter_message(
                response="utter_SKU_washout_requested",
                previous_product_number=previous_product_number,
                next_product_number=next_product_number,
                washout_duration=washout_duration,
                washout_pressure=washout_pressure,
                washout_temperature=washout_temperature,
                washout_cycles=washout_cycles)

        return []


class actionSKUWashoutCompleted(Action):
    """
        After user receives requested information the bot resets the slots.
    """

    def name(self) -> Text:
        return "action_SKU_washout_request_completed"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        # TODO write the defect information to the database
        # TODO user information to get from user profile
        # TODO try to find the defect code from DCS, if not available
        # TODO then get the component + defect from the slots

        #dispatcher.utter_message(response="utter_new_SKU_washout_registered")

        return [SlotSet("next_product_number", None),
                SlotSet("previous_product_number", None),]

