from typing import Text, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict
import time
import datetime
from datetime import datetime
# from rasa_sdk.forms import FormValidationAction

#import method to query user location
from actions.actions_cognitive_advisor.actions_zed2 import (
    zed2Interface
    )

# Needed to import synonym helper function
from actions.helpers.synonym_manager import (
    find_synonym_for_agnostic_entity
    )
# Needed to interact with graphql database
from actions.graphql_interfaces.graphql_queries import (
    graphql_searchdoc
    )

# Needed to import graphql helper functions
import actions.helpers.graphql_helpers as helpers

import logging
logger = logging.getLogger(__name__)

class actionRequestDocuments(Action):
    """
        once the form is filled, the bot retrieves relevant documents from neo4j
    """

    def name(self) -> Text:
        return "action_request_documents"

    def run(
            self,
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: DomainDict,
    ) -> List[EventType]:

        #product_number = tracker.get_slot("product_number")
        machine_component_long = tracker.get_slot('machine_component')
        machine_component = machine_component_long.replace('MC','')
        expertise_level = tracker.get_slot("expertise_level")
        production_phase = tracker.get_slot("production_phase")
        machineCompName=machine_component
        img = "empty"

        documents_info = graphql_searchdoc(machineCompName)
        documents = helpers.get_documents(documents_info)
        flag = True
        try:
            hits = len(documents)
            for doc in documents:
                document_id = doc["nameDoc"]
                document_name = doc["docDescriptionEn"]
                document_type = doc["docType"]
                document_url = doc["docUrlNl"]
                document_version = doc["docVersion"]
                document_time = doc["timeRequired"]
                img = f"{document_url}/preview"

                if flag == True:
                    text = f"Ik heb document {document_id}, {document_name}, voor asset {machine_component} gevonden"
                    dispatcher.utter_message(text = text, image = img)
                    if hits>1:
                        text = f"Verder heb ik nog {hits-1} documenten gevonden"
                        dispatcher.utter_message(text)

                else:
                    text = f'<speak><sub alias="">Document {document_id}, {document_name}: {img}</sub></speak>'
                    dispatcher.utter_message(text = text)

                flag = False
        except:
            text = f"Ik heb geen documenten voor {machine_component} gevonden"
            dispatcher.utter_message(text)

            
        return [SlotSet("machine_component", None)]

