from typing import Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.events import SlotSet, EventType
from rasa_sdk.types import DomainDict

#from requests.exceptions import ConnectionError
# from actions.helpers.lab_sim_query import (
#    get_status
#    )

from datetime import datetime, timedelta

import logging
log = logging.getLogger("Provide line status started")

from actions.helpers.PreciaDataQuery import (
   PreciaDataQuery
   )
from actions.helpers.PlotMaker import (
   PlotMaker
   )

class ProvideLineStatus(Action):
    """
       Query line status and provide it to the user
    """
    def name(self) -> Text:
        return "action_provide_line_status"

    def run(
        self,
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: DomainDict
    ) -> List[EventType]:
        '''
            Uses the tracker to get the required slots
        '''
        # todo: more informative try except
        try:
            precia = PreciaDataQuery()

            # get current time
            now = datetime.now()

            # hours of data to process for stats
            hours = float(tracker.get_slot("time_stats"))

            time_stats_offset = float(tracker.get_slot("time_stats_offset"))

            # add time window and offset to get hours to start time
            start_hours = hours + time_stats_offset

            startdatetime =  now - timedelta(hours=start_hours)

            enddatetime = now - timedelta(hours=time_stats_offset)

            # format the datetime object as a string
            enddatetime = enddatetime.strftime('%Y-%m-%d %H:%M:%S.%f')

            # format the datetime object as a string
            startdatetime = startdatetime.strftime('%Y-%m-%d %H:%M:%S.%f')

            # rough estimate of the number of rows needed for the specified hours (for when using the precia direct query)
            # limit = 2500*hours

            # query precia web api directly
            # try:
            # for querying the precia data directly from web api
            # json_data = precia.query_direct_precia(offset=0,limit=limit)

            # querying precia data via graphql and mongodb
            data = precia.query_precia_time_intervals(startdatetime,enddatetime)

            if len(data) == 0:
                dispatcher.utter_message(response="utter_no_data_timeframe")
                dispatcher.utter_message("Dus ik heb de laatste 1000 flakons gepakt")
                data = precia.query_direct_precia(offset=0,limit=1000)

            # process the data
            data,batches,products,cans,total_downtime,availability,average_speed,changeover_time,num_downtime_under_180,downtime_under_180,uptime = precia.process_precia_data(data,hours)
            if len(data)>2:
                quality = 100
                performance = int((average_speed/100)*100)
                oee = round(((quality*performance*availability)/10000),1)
                unplanned_downtime = round((total_downtime-changeover_time)/3600,1)
                changeover_time = round(changeover_time/3600,1)
                uptime = round(uptime/3600,1)
                batches = len(batches)
                # if status_response:
                dispatcher.utter_message(response="utter_line_status",
                                        oee = oee,
                                        availability = availability,
                                        performance=performance,
                                        quality=quality,
                                        cans=cans,
                                        batches=batches,
                                        average_speed=average_speed,
                                        uptime=uptime,
                                        unplanned_downtime=unplanned_downtime,
                                        changeover_time=changeover_time,
                                        num_downtime_under_180=num_downtime_under_180,
                                        downtime_under_180=downtime_under_180
                                        )
                # else:
                #     dispatcher.utter_message(
                #         response="connection_error",
                #         service_name="database"
                #     )

                plotter = PlotMaker()
                image_name = "my_plot.jpeg"
                image_link = plotter.create_and_upload(data,image_name)

                dispatcher.utter_message(
                    text=" ... en een grafiek", 
                    image = image_link
                    )

                # fig = plotter.create_plot(data)
                # plotter.save_plot(fig,"my_plot.png")
                # dispatcher.utter_message("Er ging iets mis bij het ophalen van de data")
            else:
                    dispatcher.utter_message(
                        response="utter_no_data_timeframe",
                    )

        except:
            dispatcher.utter_message(
                    response="utter_no_data_timeframe",
                )

        return [SlotSet("time_stats", None),
                SlotSet("time_stats_offset", None)]