from typing import Any, Text, Dict, List

from rasa_sdk import Action, Tracker
from rasa_sdk.executor import CollectingDispatcher
from rasa_sdk.types import DomainDict
from rasa_sdk.events import (
    SlotSet,
    EventType,
    ActionExecuted,
    SessionStarted
)

import psycopg2
import os

import logging
logger = logging.getLogger(__name__)

class zed2Interface:
    """Query users location from zed2 database."""
    def __init__(self):
        # when in docker network
        self.host_string = "api_database_1" if not os.getenv('host_string') else os.getenv('host_string')
        # when outside docker network but still local:
        # self.host_string = "local_host" if not os.getenv('host_string') else os.getenv('host_string')
        # when remote:
        # self.host_string = "[confidential]" if not os.getenv('host_string') else os.getenv('host_string')
        # use host_string = "local_host" when outside the docker network, otherwise "api_database_1"
        self.username_string = "postgres" if not os.getenv('username_string') else os.getenv('username_string')
        self.password_string = "postgres" if not os.getenv('password_string') else os.getenv('password_string')
        # when in docker network:
        self.port_string = "[confidential]" if not os.getenv('port_string') else os.getenv('port_string')
        # when outside docker network:
        # self.port_string = "[confidential]" if not os.getenv('port_string') else os.getenv('port_string')
        self.dbname_string = "coala_lab" if not os.getenv('dbname_string') else os.getenv('dbname_string')
        self.table_name_string = "zdash" if not os.getenv('table_name_string') else os.getenv('table_name_string')
    
    def query_user_location(self) -> Text:
        try:
            conn = psycopg2.connect(host = self.host_string, port = self.port_string, database = self.dbname_string, user = self.username_string, password = self.password_string, options='-c statement_timeout=1s')
            # Create a cursor object
            cur = conn.cursor()
            cur.execute("""Select "Activity", "Timestamp" FROM public.zdash ORDER BY "Timestamp" DESC LIMIT 1""")
            result = cur.fetchall()     
            cur.close()
            conn.close()
            logger.info(f"this is the user activity from {result[0][0]}")
            return result
        except:
            logger.info("I am unable to connect to the database")
            return []