
import logging
log = logging.getLogger("query zed2 data")

from datetime import datetime
import pandas as pd

import actions.helpers.config_loader as config
import plotly.graph_objects as go
import plotly.io as pio
import pyimgur
# todo remove after debug
import random
import datetime

# from PIL import Image
# import matplotlib.pyplot as plt

class PlotMaker:
    def __init__(self):
        self.client_id = config.get_credentials("imgur").get("client_id", "")
        self.client_secret = config.get_credentials("imgur").get("client_secret", "")

    # to generate data for debug
    def generate_df(self):
        start_date = datetime.datetime(2022, 1, 1, 0, 0, 0)
        delta_list = [datetime.timedelta(seconds=5), datetime.timedelta(seconds=50), datetime.timedelta(seconds=300)]
        batch_info_list = ['Batch A', 'Batch B', 'Batch C']

        df = pd.DataFrame(columns=['speed', 'delta', 'batchinformation', 'weighingdate'])

        for i in range(10):
            delta = random.choice(delta_list)
            speed = random.randint(0, 60)
            batch_info = batch_info_list[i//5]
            date = start_date + datetime.timedelta(seconds=i*10)
            if i == 0:
                delta_seconds = 0
            else:
                delta_seconds = (date - df.iloc[-1]['weighingdate']).total_seconds()
            df = df.append({'speed': speed, 'delta': delta_seconds, 'batchinformation': batch_info, 'weighingdate': date}, ignore_index=True)

        return df

    def create_plot(self, df):
        # Create a list of color strings based on the 'delta' column

        color_list = df['delta'].apply(lambda x: 'orange' if x > 5 else 'green').tolist()

        df['rolling_speed_avg'] *= 60.0

        # Create a new column 'BatchChange' that indicates when batchinformation changes
        df["BatchChange"] = (df["batchinformation"] != df["batchinformation"].shift()).astype(int)
     
        # Create the figure and set the layout
        fig = go.Figure()

        # Merge consecutive data points with the same color into a single trace
        color_trace_pairs = []
        current_color = color_list[0]
        current_trace_x = [df.loc[0, "weighingdate"]]
        current_trace_y = [df['rolling_speed_avg'].max()]

        for i in range(1, len(df)):
            if color_list[i] != current_color:
                color_trace_pairs.append((current_color, current_trace_x, current_trace_y))
                current_color = color_list[i]
                current_trace_x = [df.loc[i, "weighingdate"]]
                current_trace_y = [df['rolling_speed_avg'].max()]
            else:
                current_trace_x.append(df.loc[i, "weighingdate"])
                current_trace_y.append(df['rolling_speed_avg'].max())
        color_trace_pairs.append((current_color, current_trace_x, current_trace_y))

        # Add shaded areas
        for color, trace_x, trace_y in color_trace_pairs:
            fig.add_trace(
                go.Scatter(
                    x=trace_x + trace_x[::-1], 
                    y=trace_y + [0] * len(trace_y), 
                    mode="lines", 
                    fill="toself", 
                    fillcolor=color, 
                    line=dict(color=color, width=0),
                    showlegend=False
                )
            )

        # Add BatchChange vertical lines
        for i in range(1, len(df)):
            if df.loc[i, "BatchChange"] == 1:
                fig.add_trace(go.Scatter(x=[df.loc[i, "weighingdate"], df.loc[i, "weighingdate"]], y=[df["rolling_speed_avg"].min(), df["rolling_speed_avg"].max()], mode="lines", line=dict(color="blue"), showlegend=False))

        # Add speed line
        fig.add_trace(go.Scatter(x=df["weighingdate"], y=df["rolling_speed_avg"], mode="markers", marker=dict(color="black", size=10), showlegend=False))

        fig.update_layout(
                width=2100,
                height=1500,
                xaxis=dict(
                    tickfont=dict(
                        size=50  # Set the font size of the tick labels
                    )
                ),
                yaxis=dict(
                    tickfont=dict(
                        size=50  # Set the font size of the tick labels
                    )
                )
            )
        
        return fig

    def save_plot(self, fig, filename):
        # Save the plot as a jpeg file
        pio.write_image(fig, filename)

    def upload_to_imgur(self, filename):
        # Upload the image to Imgur
        im = pyimgur.Imgur(self.client_id)
        uploaded_image = im.upload_image(filename)

        # Get the link to the uploaded image
        img_link = uploaded_image.link

        # Return the link to the uploaded image
        return img_link

    def create_and_upload(self, df, filename):
        # Create the plot
        fig = self.create_plot(df)

        # Save the plot as a jpeg file
        self.save_plot(fig, filename)

        # Upload the image to Imgur
        img_link = self.upload_to_imgur(filename)

        # return the link to the uploaded image
        return img_link

