import base64
import json
import pickle
import requests
from requests.structures import CaseInsensitiveDict

from actions.helpers.UBI_token_manager import (
   TokenManager
   )

import actions.helpers.config_loader as config

import logging
logger = logging.getLogger(__name__)

def create_json_instance(speed,WeightNet,density,batch_num,maxSpeed,avgSpeed,deltaMaxSpeed,model_id='rf_2023_03_04_15_05_31',explanations=5):

    # Define the dictionary of feature names and values
    features = {
        "speed": str(speed),
        "WeightNet": str(WeightNet),
        "density": str(density),
        "batch_num": str(batch_num),
        "maxSpeed": str(maxSpeed),
        "avgSpeed": str(avgSpeed),
        "deltaMaxSpeed": str(deltaMaxSpeed),
    }

    # Define the JSON object
    json_object = {
        "model_id": model_id,
        "explanations": explanations,
        "explanation_type": "A",
        "instance": []
    }

    # Convert the dictionary of features into the list of instances
    for feature_name, feature_value in features.items():
        instance = {
            "feature_name": feature_name,
            "feature_value": feature_value,
            "feature_available_for_explanation": "True"
        }
        json_object["instance"].append(instance)

    return(json_object)


def parse_explanation(explain_resp):
        
    try:
        data = json.loads(explain_resp)

        logger.info("why engine response: "+str(data))

        result = data.get('data', {}).get('result', {}).get('values', {})

        classification_value = result.get('classification', {}).get('classification_value', None)
        explanation_data = result.get('explanation', [])

        explanation = []
        for features in explanation_data:
            features_list = []
            for feature in features:
                feature_name = feature.get('feature_name', None)
                feature_value = feature.get('feature_value', None)
                feature_interval = feature.get('interval', None)
                if feature_name and feature_value and feature_interval:
                    features_list.append({'feature_name': feature_name, 'feature_value': feature_value, 'feature_interval': feature_interval})
            if features_list:
                explanation.append(features_list)

    except Exception as e:
        logger.info("An error occurred: "+str(e))

    return [classification_value, explanation]

    # for debugging
    # print('Engine:', engine)
    # print('Model ID:', model_id)
    # print('Explanation Type:', explanation_type)
    # print('Classification Name:', classification_name)
    # print('Classification Value:', classification_value)
    # print('Status:', status)
    # print('Status Description:', status_description)
    # print('Explanation:', explanation)

class WhyEngineClient:
    def __init__(self):
        self.url = config.get_endpoint("graphql_why_engine").get("url", "")
        self.headers = CaseInsensitiveDict()
        self.headers["Content-Type"] = "application/json"

    def upload_model(self):
        model_data = str(base64.b64encode(open('actions/helpers/why_engine/v1RF.pkl', 'rb').read()), "utf-8")
        model_json_data = str(base64.b64encode(open('actions/helpers/why_engine/v1RF.json', 'rb').read()), "utf-8")

        data = '{"operationName": "whyEngine", ' \
           '"query": "query whyEngine($request: JSON) { result: whyEngine(request: $request) { values}}}", ' \
           '"variables": { "request": { "fileToProcess": "%s", "fileParameters": "%s", "operation": "Load" }}}'
        data = data % (model_data, model_json_data)

        load_resp = requests.post(self.url, headers=self.headers, data=data)

        if load_resp.status_code != 200:
            error_message = 'Error code: ' + str(load_resp.status_code) + ', message: ' + load_resp.text
            logger.info("Error when uploading model: "+str(error_message))
            raise Exception(error_message)

    def explain_instance(self, instance_json):
        headers = self.headers

        # Convert JSON object to a string
        instance_string = json.dumps(instance_json)

        # Encode the string using base64
        instance_encoded = base64.b64encode(instance_string.encode('utf-8'))

        # Convert the bytes to a string
        instance_data = instance_encoded.decode('utf-8')

        data = '{"operationName": "whyEngine", ' \
            '"query": "query whyEngine($request: JSON) { result: whyEngine(request: $request) { values}}}", ' \
            '"variables":{"request":{"fileToProcess":"%s", "operation":"Explain"}}}'
        data = data % instance_data

        explain_resp = requests.post(self.url, headers=headers, data=data)

        if explain_resp.status_code != 200:
            error_message = 'Error code: ' + str(explain_resp.status_code) + ', message: ' + explain_resp.text
            print(error_message)
            raise Exception(error_message)

        return explain_resp.text