from typing import Any, Dict, Text
import yaml
import logging
# import os

log = logging.getLogger("Rasa config loader")


def get_credentials(service: Text) -> Dict[Text, Any]:
    ''' Returns the credentials for the service '''

    # log.debug(os.getcwd())
    # log.debug(os.listdir())

    with open("actions/configs/credentials.yaml", "r") as stream:
        try:
            return yaml.safe_load(stream).get(service, {})
        except yaml.YAMLError as exc:
            log.error(exc)


def get_endpoint(service: Text) -> Dict[Text, Any]:
    ''' Returns the endpoint data for the service '''

    with open("actions/configs/endpoints.yaml", "r") as stream:
        try:
            return yaml.safe_load(stream).get(service, {})
        except yaml.YAMLError as exc:
            log.error(exc)


def get_action_server_config() -> Dict[Text, Any]:
    ''' Returns the action server configuration data '''

    with open("actions/configs/action_server_config.yaml", "r") as stream:
        try:
            return yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            log.error(exc)


def get_language() -> Text:
    ''' Returns the action server's language code'''

    return str(get_action_server_config().get('language', 'en'))
