'''
This helper updates the files in the actions/configs directory with the
parameters passed to this file via a shell script in a CI/CD job.

Volumes that mount these config files into the action server container
will override the files created with this script.

'''

import yaml
import sys
import logging

log = logging.getLogger("Credentials builder")

# Ensures this code is not executed when starting the action server.
if __name__ == '__main__':

    # TODO Add the graphql endpoint config here and as CI/CD variables
    gitlab_token = sys.argv[1]
    rasa_x_host_name = sys.argv[2]
    rasa_x_password = sys.argv[3]
    rasa_x_user_name = sys.argv[4]

    ###################################################################
    # Endpoints file
    ###################################################################
    file_name = "../configs/endpoints.yaml"

    with open(file_name, 'r') as f:
        try:
            doc = yaml.safe_load(f)
        except yaml.YAMLError as exc:
            log.error(exc)

    # Mapping of endpoint data and CI/CD variables
    doc['rasa-x']['host_name'] = rasa_x_host_name

    with open(file_name, 'w') as f:
        try:
            yaml.safe_dump(doc, f, default_flow_style=False)
        except yaml.YAMLError as exc:
            log.error(exc)

    print("Endpoints file updated")

    ###################################################################
    # Credentials file
    ###################################################################
    file_name = "../configs/credentials.yaml"

    with open(file_name, 'r') as f:
        try:
            doc = yaml.safe_load(f)
        except yaml.YAMLError as exc:
            log.error(exc)

    # Mapping of credentials and CI/CD variables
    doc['rasa-x']['password'] = rasa_x_password
    doc['rasa-x']['user_name'] = rasa_x_user_name
    doc['gitlab']['token'] = gitlab_token

    with open(file_name, 'w') as f:
        try:
            yaml.safe_dump(doc, f, default_flow_style=False)
        except yaml.YAMLError as exc:
            log.error(exc)

    print("Credentials file updated")
