import actions.helpers.config_loader as config
import logging
from typing import Text
import requests
from requests.exceptions import ConnectionError

class Issue():
    '''
        Representation of an issue in an issue manager,
        such as Gitlab Issues or JIRA.
    '''

    def __init__(self, title: Text, description: Text):
        self.token = config.get_credentials("gitlab").get("token", "")
        self.host_name = config.get_endpoint("gitlab").get("host_name", "")
        self.project_id = config.get_endpoint("gitlab").get("project_id", "")
        self.title = title
        self.description = description

    def send(self) -> requests.Response:
        '''
            Uses the issue manager's API to create an issue.
        '''
        log = logging.getLogger("Gitlab ticket creation")

        url = f"https://{self.host_name}/api/v4/projects/{self.project_id}/issues"
        payload = {'title': self.title, 'description': self.description}
        headers = {'PRIVATE-TOKEN': self.token}

        try:
            response = requests.post(url=url, params=payload, headers=headers)

        except ConnectionError as e:

            log.warn("No response from issue manager." +
                     "Check credentials and endpoint health.")
            log.error(e)

        return response