from cgi import test
from typing import Any, Text, Dict, List
from neo4j import Query
from rasa_sdk.events import SlotSet

from rasa_sdk.executor import CollectingDispatcher

# Needed to interact with graphql database
import json

# Needed to interact with graphql database
from actions.graphql_interfaces.graphql_queries import (
    graphql_searchdoc,
    #graphql_getcause,
    #graphql_searchset
    #graphql_get_product_profile,
    #graphql_get_user_profile,
    #graphql_get_searchsets,
    #graphql_get_documents
    )

# Needed to import synonym helper function
from actions.helpers.synonym_manager import (
    find_synonym_for_agnostic_entity
    )

import logging
log = logging.getLogger("graphql helpers debugging")

def get_solution(query_result: Text) -> Dict[Text, Text]:
    objects = json.loads(query_result)
    # objects =  query_result
    results=[]
    for item in objects['data']['findsolution']:
        descEn2 = item['descEn2']
        descNl2 = item['descNl2']
        rate = item['avrate']
        status = item['status']

        results_data = {"descEn2":descEn2,
                        "descNl2":descNl2,
                        "rate":rate,
                        "status":status
                        }
        results.append(results_data)
    return results

def get_cause(query_result: Text) -> Dict[Text, Text]:
    objects = json.loads(query_result)
    # objects =  query_result
    results=[]
    for item in objects['data']['findcause']:
        descriptionEnglish = item['descriptionEnglish']
        descriptionDutch = item['descriptionDutch']

        results_data = {"descriptionEnglish":descriptionEnglish,
                        "descriptionDutch":descriptionDutch,
                        }
        results.append(results_data)
    return results


def get_sku_settings(query_result: Text) -> Dict[Text, Text]:
    objects = json.loads(query_result)
    settings=[]
    for item in objects['data']['searchset']:
        comments = item['comments']
        fillingPressure = item['fillingPressure']
        fillingSpeed = item['fillingSpeed']
        rate = item['avrate']
        status = item['status']
        #try:
        datetime = item['datetime']
        name = item['name']
        #except IndexError:
        #    datetime = None
        setting_data = {"fillingPressure":fillingPressure,
                        "fillingSpeed":fillingSpeed,
                        "comments":comments,
                        "datetime":datetime,
                        "rate":rate,
                        "status":status,
                        "name":name
                        }
        settings.append(setting_data)
        
    return settings

def get_documents(query_result: Text) -> Dict[Text, Text]:
    objects = json.loads(query_result)
    documents = []
    for item in objects['data']['searchdoc']:
        nameDoc = item['nameDoc']
        docDescriptionEn = item['docDescriptionEn']
        docType = item['docType']
        docUrlNl = item['docUrlNl']
        docVersion = item['docVersion']
        timeRequired = item['timeRequired']
        rate = item['avrate']
        status = item['status']

        documents_details = {"nameDoc":nameDoc,
                    "docDescriptionEn":docDescriptionEn,
                    "docType":docType,
                    "docUrlNl":docUrlNl,
                    "docVersion":docVersion,
                    "timeRequired":timeRequired,
                    "rate":rate,
                    "status":status
                    }
        documents.append(documents_details)

    return documents
 
# # BIBA helper function for reference
# def get_checklist_data(query_result: Text) -> Dict[Text, Text]:
#     '''
#         This function reads the checklist information for one step
#         from the json object recieved from graphql.
#     '''
#     objects = json.loads(query_result)

#     checklist_info_for_step = objects['data']['users'][0]['checklists'][0]
#     first_step = checklist_info_for_step['first_step']
#     last_step = checklist_info_for_step['last_step']
#     try:
#         step_info = checklist_info_for_step['steps'][0]
#         step_time = step_info['time']
#         next_step = step_info['next_step'][0]['step']
#         current_step_component = step_info['components'][0]['component']
#         current_step_objects = step_info['components'][0]['objects'][0]
#         current_step_objective = current_step_objects['objective']
#         current_step_action = current_step_objects['actions'][0]['action']
#         current_step_defect = current_step_objects['defects'][0]['defect']
#     except IndexError:
#         step_info = None
#         step_time = None
#         next_step = None
#         current_step_component = None
#         current_step_objects = None
#         current_step_objective = None
#         current_step_action = None
#         current_step_defect = None

#     return {
#         "first_step": first_step,
#         "last_step": last_step,
#         "step_time": step_time,
#         "next_step": next_step,
#         "current_step_component": current_step_component,
#         "current_step_objective": current_step_objective,
#         "current_step_action": current_step_action,
#         "current_step_defect": current_step_defect,
#         }


# # BIBA helper function for reference
# def get_active_checklist_step_data(query_result: Text) -> Dict[Text, Text]:
#     '''
#         This function reads the checklist information for one step
#         from the json object recieved from graphql.
#     '''
#     objects = json.loads(query_result)

#     active_checklist_info = objects['data'][
#         'get_product_profile_active_checklist_step'][0]["checklists"][0]
#     first_step = active_checklist_info['first_step']
#     last_step = active_checklist_info['last_step']

#     try:
#         active_step_info = active_checklist_info['steps'][0]
#         active_step_number = active_step_info["step"]
#         log.info(f"active step number is {active_step_number}")
#         active_step_time = active_step_info['time']
#         log.info(f"active step time is {active_step_time}")
#         next_step = active_step_info['next_step'][0]['step']
#         previous_step = active_step_info['previous_step'][0]['step']
#         active_step_component = active_step_info['components'][0]['component']
#         log.info(f"active step component is {active_step_component}")
#         active_step_objects = active_step_info['components'][0]['objects'][0]
#         active_step_objective = active_step_objects['objective']
#         active_step_action = active_step_objects['actions'][0]['action']
#         active_step_defect = active_step_objects['defects'][0]['defect']

#     except IndexError:
#         active_step_number = None
#         active_step_time = None
#         next_step = None
#         previous_step = None
#         active_step_component = None
#         active_step_objective = None
#         active_step_action = None
#         active_step_defect = None

#     return {
#         "first_step": first_step,
#         "last_step": last_step,
#         "active_step_number": active_step_number,
#         "active_step_time": active_step_time,
#         "next_step": next_step,
#         "previous_step": previous_step,
#         "active_step_component": active_step_component,
#         "active_step_objective": active_step_objective,
#         "active_step_action": active_step_action,
#         "active_step_defect": active_step_defect,
#         }

# # BIBA helper function for reference. We may need something similar.
# def get_user_profile(dispatcher: CollectingDispatcher,
#                      user_id: Text) -> List[Dict[Text, Any]]:
#     '''
#         This function reads the profile of one specific user
#         and saves the arrtibutes in slots
#     '''

#     user_profile_result = graphql_get_user_profile(
#         user_id
#     )

#     # Handles error messages in graphql query results.
#     if 'errors' in user_profile_result:
#         errors: Dict = user_profile_result.get('errors')

#         for error in errors:
#             dispatcher.utter_message(
#                 response=error.get('error_response'),
#                 service_name=error.get('service_name')
#             )
#         return []

#     objects = json.loads(user_profile_result)
#     user_profile = objects['data']['user_profile'][0]

#     try:
#         user_expertise_level = user_profile['user_expertise_level']
#         user_language_code = user_profile['user_language']
#         user_language = find_synonym_for_agnostic_entity(
#                         user_language_code)["synonym_for_entity"]

#     except IndexError:
#         user_expertise_level = None
#         user_language = None

#     return [
#             SlotSet("user_expertise_level", user_expertise_level),
#             SlotSet("user_language", user_language)
#         ]


# def validate_product_identifier(dispatcher: CollectingDispatcher,
#                                 product_id: Text) -> bool:
#     '''
#         This function checks if a product ID exists in the databse.
#         The database returns an empty array if there is no such product.
#     '''

#     product_profile_result = graphql_get_product_profile(
#             product_id)

#     # Handles error messages in graphql query results.
#     if 'errors' in product_profile_result:
#         errors: Dict = product_profile_result.get('errors')

#         for error in errors:
#             dispatcher.utter_message(
#                 response=error.get('error_response'),
#                 service_name=error.get('service_name')
#             )

#         return False

#     objects = json.loads(product_profile_result)
#     product = objects['data']['product_profile']
#     log.info(product)

#     try:
#         product_identifier = product[0]['product_identifier']
#         log.info(product_identifier)
#     except IndexError:
#         return False

#     return True
