# This code is a demo code for websockets only.

from os import error
from typing import Any, Dict, Text

import socketio

# Example from here https://github.com/miguelgrinberg/python-socketio/tree/main/examples/client/threads

sio = socketio.Client()

'''
    This file contains methods to send the utterance to Rasa the socket.io endpoint.

'''


@sio.event
def connect():
    print('connected to server')


@sio.event
def disconnect():
    print('disconnected from server')


@sio.event
def user_uttered(message):
    '''Receives the user message, processes it, and returns the result'''
    print(message)
    return message


@sio.event
def bot_uttered(message: Dict[Text, Any]) -> None:
    '''
    This method decides how to respond to an incoming Rasa message.
    The method's name must be equal to the event name for the socket.io
    channel configured in Rasa's credentials.yml
        socketio:
        user_message_evt: user_uttered
        bot_message_evt: bot_uttered <--
        session_persistence: true
    '''
    print(message)


if __name__ == '__main__':
    host_name = "localhost"
    port = "5005"

    try:
        sio.connect(f"http://{host_name}:{port}/socket.io", transports='websocket')

    except error as e:
        print(f"failing: {e}")

    else:
        # Emit event to continue a Rasa session.
        sio.emit("session_request", {"session_id": "user1234"})

    # Emit event that user uttered a message and wait for the bots response.
    metadata = {'data': {'id': 12345, 'description': 'Text description'}}

    sio.call(
        event="user_uttered",
        data={
            "message": user_uttered("Hello world"),
            "session_id": "user1234",
            "metadata": metadata
            }
        )

    sio.wait()
